/****************************************************************************
 *
 * Copyright (c) 2008-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
 
package org.ow2.easywsdl.wsdl.decorator;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBindingFault;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBindingInput;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBindingOperation;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBindingOutput;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBinding.StyleConstant;
import org.ow2.easywsdl.wsdl.api.binding.BindingProtocol.SOAPMEPConstants;

/**
 * @author Nicolas Salatge - EBM WebSourcing
 */
@SuppressWarnings("unchecked")
public abstract class DecoratorBindingOperationImpl<O extends AbsItfOperation, BIn extends AbsItfBindingInput, BOut extends AbsItfBindingOutput, BF extends AbsItfBindingFault> 
extends Decorator<AbsItfBindingOperation<O, BIn, BOut, BF>> {

    /**
	 *
	 */
    private static final long serialVersionUID = 1L;

    public DecoratorBindingOperationImpl(final AbsItfBindingOperation<O, BIn, BOut, BF> bindingOperation)
            throws WSDLException {
        this.internalObject = bindingOperation;
    }

    public void addFault(final BF arg0) {
        this.internalObject.addFault(arg0);
    }

    public BF getFault(final String arg0) {
        return this.internalObject.getFault(arg0);
    }

    public List<BF> getFaults() {
        return this.internalObject.getFaults();
    }

    public String getHttpContentEncodingDefault() {
        return this.internalObject.getHttpContentEncodingDefault();
    }

    public String getHttpFaultSerialization() {
        return this.internalObject.getHttpFaultSerialization();
    }

    public String getHttpInputSerialization() {
        return this.internalObject.getHttpInputSerialization();
    }

    public String getHttpLocation() {
        return this.internalObject.getHttpLocation();
    }

    public String getHttpMethod() {
        return this.internalObject.getHttpMethod();
    }

    public String getHttpOutputSerialization() {
        return this.internalObject.getHttpOutputSerialization();
    }

    public String getHttpQueryParameterSeparator() {
        return this.internalObject.getHttpQueryParameterSeparator();
    }

    public BIn getInput() {
        return this.internalObject.getInput();
    }

    public SOAPMEPConstants getMEP() {
        return this.internalObject.getMEP();
    }

    public O getOperation() {
        return this.internalObject.getOperation();
    }

    public BOut getOutput() {
        return this.internalObject.getOutput();
    }

    public QName getQName() {
        return this.internalObject.getQName();
    }

    public String getSoapAction() {
        return this.internalObject.getSoapAction();
    }

    public StyleConstant getStyle() {
        return this.internalObject.getStyle();
    }

    public boolean isHttpIgnoreUncited() {
        return this.internalObject.isHttpIgnoreUncited();
    }

    public BF removeFault(final String arg0) {
        return this.internalObject.removeFault(arg0);
    }

    public void setInput(final BIn arg0) {
        this.internalObject.setInput(arg0);
    }

    public void setMEP(final SOAPMEPConstants arg0) {
        this.internalObject.setMEP(arg0);
    }

    public void setOutput(final BOut arg0) {
        this.internalObject.setOutput(arg0);
    }

    public void setQName(final QName arg0) {
        this.internalObject.setQName(arg0);
    }

 	public BF createFault() {
		return this.internalObject.createFault();
	}

	public BIn createInput() {
		return this.internalObject.createInput();
	}

	public BOut createOutput() {
		return this.internalObject.createOutput();
	}

	public void setSoapAction(String action) {
		this.internalObject.setSoapAction(action);
	}

}
